import std/json, std/sequtils, std/times
import timeutils

type
  HffNotionConfigProperty = tuple[key: string, textNode: JsonNode, dateNode: JsonNode]

  HffNotionConfig* = object
    addressesDbId*: string
    familiesDbId*: string
    peopleDbId*: string
    syncRecordsDbId*: string
    lastSyncedAt*: DateTime

proc getCfgProp(cfgPages: JsonNode, key: string): HffNotionConfigProperty =

  let propNodes = cfgPages.filterIt(
    it{"properties", "Key", "title"}[0]["plain_text"].getStr == key)

  if propNodes.len != 1:
    raise newException(Exception,
      "there is no configuration value for key '" & key &
      "' in the Notion Configuration table.")

  return (
    propNodes[0]{"properties", "Key", "title"}[0]["plain_text"].getStr,
    propNodes[0]{"properties", "TextValue", "rich_text"},
    propNodes[0]{"properties", "DateTimeValue", "date"})

proc dateValue(prop: HffNotionConfigProperty): DateTime =
  if isNil(prop.dateNode) or prop.dateNode.kind == JNull:
    raise newException(Exception, prop.key & " does not have a date value")

  return parseIso8601(prop.dateNode["start"].getStr)

proc textValue(prop: HffNotionConfigProperty): string =
  if prop.textNode.len == 0:
    raise newException(Exception, prop.key & " does not have a text value")

  return prop.textNode[0]["plain_text"].getStr

proc parseHffNotionConfig*(n: JsonNode): HffNotionConfig =
  result = HffNotionConfig(
    addressesDbId: n.getCfgProp("addressesDbId").textValue,
    familiesDbId: n.getCfgProp("familiesDbId").textValue,
    peopleDbId: n.getCfgProp("peopleDbId").textValue,
    syncRecordsDbId: n.getCfgProp("syncRecordsDbId").textValue,
    lastSyncedAt: n.getCfgProp("lastSyncedAt").dateValue)
